@@ -16,14 +16,14 @@ from PyQt4.QtGui import QPlainTextEdit, QColor, \
                         QTextDocument, QKeyEvent, \
                         QMessageBox, \
                         QPalette
-                        
+
 from plugins.plugins_api import init_plugin_system, filter_plugins_by_capability
 from recent_files import RecentFiles
 
 
 class KhtTextEdit( QPlainTextEdit):
     """ Widget which handle all specifities of implemented in the editor"""
-        
+
     def __init__(self, parent=None, filename=None):
         """Initialization, can accept a filepath as argument"""
         QPlainTextEdit.__init__(self, parent)
@@ -37,45 +37,45 @@ class KhtTextEdit( QPlainTextEdit):
         self.qt18720 = False
 
         if (bool(parent.settings.value("qt18720"))):
-            self.qt18720 = True                                 
+            self.qt18720 = True
         else:
             self.qt18720 = False
-        
+
         # Brace matching
         self.bracepos = None
-        
+
         # Init scroller and area which are tricky hack to speed scrolling
         scroller = None
         area = None
-        
-        #Plugin init moved to editor_window.py        
+
+        #Plugin init moved to editor_window.py
         #initialization init of plugin system
-        #Maybe be not the best place to do it ... 
+        #Maybe be not the best place to do it ...
         #init_plugin_system({'plugin_path': '/home/opt/khteditor/plugins',
         #                    'plugins': ['autoindent']})
-        
+
         #If we have a filename
         self.filename = filename
         if (self.filename == None) or (self.filename == ''):
             self.filename = u'Unnamed.txt'
         self.document().setModified(False)
         parent.setWindowTitle(self.filename)
-        
+
         #Set no wrap
         if (bool(parent.settings.value("WrapLine"))):
-            self.setLineWrapMode( QPlainTextEdit.NoWrap)                        
+            self.setLineWrapMode( QPlainTextEdit.NoWrap)
         else:
             self.setLineWrapMode( QPlainTextEdit.WidgetWidth)
 
         font =  QFont()
         try:
-            if parent.settings.contains('FontName'):       
+            if parent.settings.contains('FontName'):
                 font.setFamily(parent.settings.value('FontName'))
             else:
                 font.setFamily("Courier")
         except:
             font.setFamily("Courier")
-            
+
         font.setFixedPitch(True)
         font.setPointSize(12)
 
@@ -87,14 +87,14 @@ class KhtTextEdit( QPlainTextEdit):
                 font.setPointSize(11)
         except:
             font.setPointSize(11)
-            
+
         #Set Font
         self.document().setDefaultFont( font)
-        
+
 
         #Remove auto capitalization
         self.setInputMethodHints(Qt.ImhNoAutoUppercase)
-        
+
         #Keep threaded plugins references to avoid them to be garbage collected
         self.threaded_plugins = []
         self.enabled_plugins = parent.enabled_plugins
@@ -115,9 +115,10 @@ class KhtTextEdit( QPlainTextEdit):
         self.setMinimumHeight(max(s.height(), s.height() + (fr.height() - cr.height() - 1)))
         self.setMinimumWidth(max(s.width(),s.width() + (fr.width()-cr.width()) - 1))
 
+
     def ensureVisible(self,pos,xmargin,ymargin):
 
-        visible = self.area.viewport().size()        
+        visible = self.area.viewport().size()
         currentPos =  QPoint(self.area.horizontalScrollBar().value(),
                       self.area.verticalScrollBar().value())
         posRect =  QRect(pos.x()-xmargin, pos.y()-ymargin,2*xmargin,2*ymargin)
@@ -139,12 +140,16 @@ class KhtTextEdit( QPlainTextEdit):
 
 
     def curPositionChanged(self):
+        #Plugin hook
+        for plugin in filter_plugins_by_capability('beforeCursorPositionChanged',self.enabled_plugins):
+            plg = plugin()
+            plg.do_beforeCursorPositionChanged(self)
+
         #Hilight current line
         #Workarround QTBUG-18720
         self.highlightCurrentLine()
-        
-        #Make sure cursor is visible
 
+        #Make sure cursor is visible
         cursor = self.cursorRect()
         pos = cursor.center()
         self.area.ensureVisible(pos.x(),pos.y(), 2*cursor.width()+20, 2*cursor.height())
@@ -184,8 +189,8 @@ class KhtTextEdit( QPlainTextEdit):
     def save(self):
         """Hum ... just save ..."""
         if self.filename.startswith("Unnamed"):
-            filename = self.parent().parent().parent().saveAsFile()  
-            if not (filename == ''):            
+            filename = self.parent().parent().parent().saveAsFile()
+            if not (filename == ''):
                 return
             self.filename = filename
         self.setWindowTitle( QFileInfo(self.filename).fileName())
@@ -225,7 +230,7 @@ class KhtTextEdit( QPlainTextEdit):
             text = self.get_text(position, 'eof')
             i_start_open = 1
             i_start_close = 1
-        else:            
+        else:
             bracemap = {')': '(', ']': '[', '}': '{'}
             text = self.get_text('sob', position)
             i_start_open = len(text)-1
@@ -257,7 +262,7 @@ class KhtTextEdit( QPlainTextEdit):
             else:
                 # no matching brace
                 return
-    
+
     def __highlight(self, positions, color=None, cancel=False):
         cursor =  QTextCursor(self.document())
         modified = self.document().isModified()
@@ -267,7 +272,7 @@ class KhtTextEdit( QPlainTextEdit):
             cursor.setPosition(position)
             cursor.movePosition( QTextCursor.NextCharacter,
                                  QTextCursor.KeepAnchor)
-            charformat = cursor.charFormat()            
+            charformat = cursor.charFormat()
             pen =  QPen(Qt.NoPen) if cancel else  QPen(color)
             charformat.setTextOutline(pen)
             cursor.setCharFormat(charformat)
@@ -275,29 +280,29 @@ class KhtTextEdit( QPlainTextEdit):
             charformat =  QTextCharFormat()
             cursor.movePosition( QTextCursor.NextCharacter,
                                  QTextCursor.KeepAnchor)
-            cursor.setCharFormat(charformat)            
+            cursor.setCharFormat(charformat)
             cursor.clearSelection()
             self.setCurrentCharFormat(charformat)
         self.document().setModified(modified)
 
-            
-    def highlightCurrentLine(self):            
+
+    def highlightCurrentLine(self):
         #Hilgight background
         _selection =  QTextEdit.ExtraSelection()
         _selection.cursor = self.textCursor()
         _selection.cursor.clearSelection()
         _selection.format.setBackground(self.hl_color)
-        _selection.format.setProperty( QTextFormat.FullWidthSelection, True)        
+        _selection.format.setProperty( QTextFormat.FullWidthSelection, True)
         extraSelection = []
         extraSelection.append(_selection)
-        
+
         #Highlight Braces
         if self.bracepos is not None:
             self.bracepos = None
-        cursor = self.textCursor()        
+        cursor = self.textCursor()
         if not cursor.position() == 0:
             cursor.movePosition( QTextCursor.PreviousCharacter,
-                                 QTextCursor.KeepAnchor)                           
+                                 QTextCursor.KeepAnchor)
             text = unicode(cursor.selectedText())
             pos1 = cursor.position()
             if text in (')', ']', '}'):
@@ -329,10 +334,10 @@ class KhtTextEdit( QPlainTextEdit):
                 _selection.format.setForeground(Qt.white)
                 _selection.cursor = cursor
                 extraSelection.append(_selection)
-                
-        if not self.qt18720:        
+
+        if not self.qt18720:
             self.setExtraSelections(extraSelection)
-                    
+
     def load(self):
         """Load ?"""
         exception = None
@@ -347,6 +352,11 @@ class KhtTextEdit( QPlainTextEdit):
             self.setPlainText(stream.readAll())
             self.document().setModified(False)
             self.setWindowTitle( QFileInfo(self.filename).fileName())
+            for plugin in filter_plugins_by_capability('afterFileOpen',self.enabled_plugins):
+                plg = plugin()
+                self.threaded_plugins.append(plg)
+                plg.do_afterFileOpen(self)
+
         except (IOError, OSError), error:
             exception = error
         finally:
@@ -374,7 +384,7 @@ class KhtTextEdit( QPlainTextEdit):
             while True:
                 whitespace = re.match(r"(\s{0,4})",
                                 str(block.text())).group(1)
-                cursor = self.textCursor() 
+                cursor = self.textCursor()
                 cursor.setPosition(block.position())
                 for i in range(len(whitespace)): #@UnusedVariable
                     cursor.deleteChar()
@@ -391,7 +401,7 @@ class KhtTextEdit( QPlainTextEdit):
         else:
             block = self.document().findBlock(maincursor.selectionStart())
             while True:
-                cursor = self.textCursor() 
+                cursor = self.textCursor()
                 cursor.setPosition(block.position())
                 cursor.insertText("    ")
                 if block.contains(maincursor.selectionEnd()):
@@ -417,17 +427,17 @@ class KhtTextEdit( QPlainTextEdit):
         # Beginning of undo block
         pcursor = self.textCursor()
         pcursor.beginEditBlock()
-    
+
         #cursor at start as we replace from start
         cursor = self.textCursor()
         cursor.setPosition(0)
-        
+
         # Replace all we can
         while True:
             # self is the QTextEdit
             if args[3]:
                 cursor=self.document().find( QRegExp(what),
-                                       cursor,flags)                
+                                       cursor,flags)
             else:
                 cursor=self.document().find(what,cursor,flags)
             if not cursor.isNull():
@@ -438,10 +448,10 @@ class KhtTextEdit( QPlainTextEdit):
                     print 'no selection'
             else:
                 break
-    
+
         # Mark end of undo block
         pcursor.endEditBlock()
-        
+
     def replace(self, what, new, *args):
         """Replace the first occurence of a search
         arg[0] ->  QTextDocument.FindCaseSensitively
@@ -463,21 +473,21 @@ class KhtTextEdit( QPlainTextEdit):
         pcursor.beginEditBlock()
 
         cursor = self.textCursor()
-    
+
         # Replace
         # self is the QTextEdit
         if args[3]==True:
             cursor = self.document().find( QRegExp(what),
-                                    cursor, flags)                
+                                    cursor, flags)
         else:
             cursor = self.document().find(what, cursor, flags)
         if not cursor.isNull():
             if cursor.hasSelection():
                 cursor.insertText(new)
-    
+
         # Mark end of undo block
         pcursor.endEditBlock()
-        
+
     def find(self, what, *args):
         """Perform a search
         arg[0] ->  QTextDocument.FindCaseSensitively
@@ -486,7 +496,7 @@ class KhtTextEdit( QPlainTextEdit):
         arg[3] ->  QTextDocument.RegEx
         """
         print 'find called'
-        
+
         # Use flags for case match
         flags =  QTextDocument.FindFlags()
         if args[0]==True:
@@ -505,8 +515,8 @@ class KhtTextEdit( QPlainTextEdit):
                                        cursor, flags)
 
         if not cursor.isNull():
-            self.setTextCursor(cursor)            
-            
+            self.setTextCursor(cursor)
+
     def duplicate(self):
         """Duplicate the current line or selection"""
         maincursor = self.textCursor()
@@ -520,17 +530,17 @@ class KhtTextEdit( QPlainTextEdit):
             block = self.document().findBlock(maincursor.selectionStart())
             line = u''
             while True:
-                cursor = self.textCursor()                                     
+                cursor = self.textCursor()
                 cursor.setPosition(block.position())
 
                 line = line + '\n' + block.text()
-                
+
                 if block.contains(maincursor.selectionEnd()):
                     break
                 block = block.next()
             cursor.movePosition( QTextCursor.EndOfBlock)
             cursor.insertText(line)
-        
+
     def gotoLine(self, line):
             print 'goto line:'+str(line)
             cursor = self.textCursor()
@@ -539,8 +549,8 @@ class KhtTextEdit( QPlainTextEdit):
             self.setTextCursor(cursor)
             self.ensureCursorVisible()
             self.parent().activateWindow()
-            
-            
+
+
     def comment(self):
         """Comment the current line or selection"""
         maincursor = self.textCursor()
@@ -555,7 +565,7 @@ class KhtTextEdit( QPlainTextEdit):
         else:
             block = self.document().findBlock(maincursor.selectionStart())
             while True:
-                cursor = self.textCursor()                                     
+                cursor = self.textCursor()
                 cursor.setPosition(block.position())
 
                 if str(block.text()).startswith('#'):
@@ -575,7 +585,7 @@ class KhtTextEdit( QPlainTextEdit):
         cursor.setPosition(position_to,  QTextCursor.KeepAnchor)
         return cursor
 
-        
+
     def get_position(self, position):
         cursor = self.textCursor()
         if position == 'cursor':
